# coding:utf-8
from educommon.utils.db import get_related_instances
from m3.db import safe_delete
from m3_django_compat import atomic
from m3_django_compat import get_model


class CascadeDeletePackMixin(object):

    u"""Mixin определяющий исключаемые поля модели при проверке DeleteCheck.

    Поскольку DeleteCheck срабатывает до вызова delete_row возникают ситуации,
    когда нам требуется при работе с этим паком исключить определенные
    поля модели, из которых имеются ссылки на модель. Для решения данных
    ситуаций используется атрибут cascade_delete_objects.

    .. code-block:: python

        cascade_delete_objects = (( 'parent', 'parentportfolio', 'person'),)
    """
    # содержит кортежи формата
    # (имя приложения, имя модели приложения, имя поля)
    cascade_delete_objects = ()

    def _cascade_delete(self, obj_id, request, context):
        for app_label, model_name, field_name in (
                self.__class__.cascade_delete_objects):
            model = get_model(app_label, model_name)
            lookup = {
                field_name + '_id': obj_id
            }
            for obj in model.objects.filter(**lookup):
                if hasattr(obj, 'safe_delete') and callable(obj.safe_delete):
                    obj.safe_delete()
                else:
                    safe_delete(obj)

    @classmethod
    def skip_field(cls, field):
        if cls.cascade_delete_objects:
            for app_label, model_name, field_name in (
                    cls.cascade_delete_objects):
                model = get_model(app_label, model_name)
                if field.name == field_name and field.model == model:
                    return True
        return False

    @atomic
    def delete_row(self, obj_id, request, context):
        self._cascade_delete(obj_id, request, context)
        return super(CascadeDeletePackMixin, self).delete_row(
            obj_id, request, context)


class CascadeDeleteMixin(object):

    u"""Mixin определяющий исключаемые поля модели при проверке DeleteCheck.

    Поскольку DeleteCheck учитывает все зависимые обьекты, возникают ситуации
    при которых нам необходимо каскадно удалить зависимые объекты. Для
    каскадного данных записей необходимо указать имена полей, на которые
    ссылаются зависимые объекты.

    .. code-block:: python

        cascade_delete_for = ('contact',)
    """

    @staticmethod
    def skip_field(field):
        return field.name in getattr(field.model, 'cascade_delete_for', ())

    @atomic
    def safe_delete(self):
        for obj in get_related_instances(
            self,
            collapse=False,
            skip_func=not self.skip_field
        ):
            print obj
            if hasattr(obj, 'safe_delete') and callable(obj.safe_delete):
                obj.safe_delete()
            else:
                safe_delete(obj)

        return super(CascadeDeleteMixin, self).safe_delete()
